module HhowmmConfig
    (
      ConfigInfo(..)
    , readConfig
    , defaultConfig
    ) where

import Data.ConfigFile
import Data.Either.Utils
import Data.List

data ConfigInfo = ConfigInfo
    { confBaseDir :: FilePath
    , confFontSize :: Int
    , confExtensions :: [String]
    , confBrowserCmd :: String
    , confEditorCmd :: String
    } deriving (Show)

defaultConfig :: ConfigInfo
defaultConfig = ConfigInfo
    { confBaseDir = "howm"
    , confFontSize = 12
    , confExtensions = [".txt"]
    , confBrowserCmd = "firefox "
    , confEditorCmd = "vi "
    }

readConfig :: FilePath -> IO ConfigInfo
readConfig file = do
    val <- readfile emptyCP file
    let cp = forceEither val
    let fs = case get cp "DEFAULT" "fontsize" of
                Right x -> read x
                Left _ -> confFontSize defaultConfig
    putStrLn $ "fontsize: " ++ show fs
    let bd = case get cp "DEFAULT" "basedir" of
                Right x -> x
                Left _ -> confBaseDir defaultConfig
    putStrLn $ "basedir: " ++ bd
    let ext = case get cp "DEFAULT" "exts" of
                Right x -> map normalizeExtension $ words x
                Left _ -> confExtensions defaultConfig
    putStrLn $ "extensions: " ++ unwords ext
    let br = case get cp "DEFAULT" "browser" of
                Right x -> if isSuffixOf " " x then x else x ++ " "
                Left _ -> confBrowserCmd defaultConfig
    putStrLn $ "browser command: " ++ br
    let ed = case get cp "DEFAULT" "editor" of
                Right x -> if isSuffixOf " " x then x else x ++ " "
                Left _ -> confEditorCmd defaultConfig
    putStrLn $ "editor command: " ++ ed

    return ConfigInfo {confBaseDir = bd, confFontSize = fs, confExtensions = ext, confBrowserCmd = br, confEditorCmd = ed}
  where
    normalizeExtension [] = []
    normalizeExtension str =
        dropWhileEnd (== ',') $ if head str == '.' then str else "." ++ str

