module HhowmmFilename
    (
      getRecursiveContents
    , filterFilenameByDays
    , filterFilenameByExts
    , newFilename
    ) where

import Control.Monad (forM, filterM)
import Data.List
import Data.Time
import System.Directory (doesDirectoryExist, getDirectoryContents)
import System.FilePath ((</>))
import System.Locale
import System.PosixCompat.Files
import System.PosixCompat.Time

getRecursiveContents :: FilePath -> IO [FilePath]
getRecursiveContents topdir = do
    names <- getDirectoryContents topdir
    let properNames = filter (`notElem` [".", ".."]) names
    paths <- forM properNames $ \name -> do
        let path = topdir </> name
        isDir <- doesDirectoryExist path
        if isDir
            then getRecursiveContents path
            else return [path]
    return (concat paths)

filterFilenameByDays :: [FilePath] -> Int -> IO [FilePath]
filterFilenameByDays [] _ = return []
filterFilenameByDays list days = do
    now <- epochTime
    let limitTime = now - (fromIntegral days*24*60*60)
    filterM (isModifiedAfter limitTime) list
  where isModifiedAfter time path = do
            status <- getFileStatus path
            if time > modificationTime status
                then return False
                else return True

filterFilenameByExts :: [FilePath] -> [String] -> [FilePath]
filterFilenameByExts [] _ = []
filterFilenameByExts list exts = concatMap (\x -> filter (isSuffixOf x) list) exts

newFilename :: IO (FilePath, FilePath)
newFilename = do
    now <- getZonedTime
    return (  formatTime defaultTimeLocale "%Y" now </> formatTime defaultTimeLocale "%m" now
            , formatTime defaultTimeLocale "%Y-%m-%d-%H%M%S.txt" now
            )

