module HhowmmGrep
    (
      GrepItem(..)
    , andGrep
    , hashtagSearch
    , grep
    , sortByModTime
    ) where

import Control.Monad
import Data.Function (on)
import Data.List
import System.Directory
import System.Environment
import System.Time
import Data.Time.Clock

import HhowmmFilename
import HhowmmDefs

data GrepItem = GrepItem
    { fileName :: FilePath
    , lineNum :: Int
    , lineText :: String
    } deriving (Show, Eq)

search :: (String -> String -> Bool) -> String -> String -> [(Int, String)]
search func pattern content =
    let linesWithNumber = zip [0..] $ lines content in
    filter (\(_, lineStr) -> func pattern lineStr) linesWithNumber

processFile :: (String -> String -> Bool) -> String -> FilePath -> IO [GrepItem]
processFile func pattern file = do
    exists <- doesFileExist file
    if not exists
        then return []
        else do
            content <- readFile file
            return (map (\(n, text) -> GrepItem file n text) (search func pattern content))

grep :: (String -> String -> Bool) -> String -> [FilePath] -> IO [GrepItem]
grep _ [] _ = return []
grep _ _ [] = return []
grep func pattern files = do
    results <- forM files $ processFile func pattern
    return (concat results)

listFilesInResults :: [GrepItem] -> [FilePath]
listFilesInResults [] = []
listFilesInResults xs = nub $ map fileName xs

mergeResults :: [GrepItem] -> [GrepItem] -> [GrepItem]
mergeResults [] ys = ys
mergeResults _ []  = []
mergeResults xs ys = nub $ filter (\item1 -> any (\item2 -> fileName item1 == fileName item2) ys) xs ++ ys

andGrep' :: (String -> String -> Bool) -> [String] -> [FilePath] -> [GrepItem] -> IO [GrepItem]
andGrep' _ [] _ results = return results
andGrep' func (p:pats) files results = do
    tmpResults <- grep func p files
    let newResults = mergeResults results tmpResults
    andGrep' func pats (listFilesInResults newResults) newResults

andGrep :: [String] -> [FilePath] -> IO [GrepItem]
andGrep _ [] = return []
andGrep [] files = titleSearch files
andGrep patterns files = andGrep' isInfixOf patterns files []

readFirstLine :: FilePath -> IO String
readFirstLine file = do
    exists <- doesFileExist file
    if not exists
        then return ""
        else do
            content <- readFile file
            if null content
                then return ""
                else return $ head $ lines content

titleSearch :: [FilePath] -> IO [GrepItem]
titleSearch files = do
    results <- mapM (\file -> do
                        res <- processFile isPrefixOf title_str file
                        if null res
                            then do
                                str <- readFirstLine file
                                return [GrepItem file 0 str]
                            else return res
                    ) files
    return $ concat results

searchNearestTitle :: [GrepItem] -> GrepItem -> GrepItem
searchNearestTitle titles target =
    if null befores
        then target
        else last befores
  where
    befores = takeWhile (\itm -> lineNum itm < lineNum target)
                $ sortBy (compare `on` lineNum)
                $ filter (\itm -> fileName itm == fileName target) titles

hashtagSearch :: String -> [FilePath] -> IO [GrepItem]
hashtagSearch _ [] = return []
hashtagSearch [] _ = return []
hashtagSearch tag files = do
    titles <- titleSearch files
    tags <- grep isPrefixOf tag files
    return $ map (searchNearestTitle titles) tags

sortByModTime :: [GrepItem] -> IO [GrepItem]
sortByModTime list = do
    listWithModTime <- mapM appendModTime list
    return $ map fst $ sortBy (\(_, t1) (_, t2) -> compare t2 t1) listWithModTime
  where appendModTime :: GrepItem -> IO (GrepItem, UTCTime)
        appendModTime x = do
            t <- getModificationTime $ fileName x
            return (x, t)
